-------------------------------------------------------------------------------
--
-- File: NoiseFilterD.vhd
-- Author: William Zhang 
-- Original Project: FPGA Benchmark
-- Date: 2007.06.01
--
-------------------------------------------------------------------------------
--
-- (c) 2007 Copyright Broadband Transmission Lab
-- All Rights Reserved
-- EE Dept. at Tsinghua Univ.
--
-------------------------------------------------------------------------------
--
-- Purpose: 
-- Filter the out band noise. The filter uses the direct structure. The DC gain 
-- is 1.
-- 
--  -------       -------      -------      -------      -------       -------
--  |  -1 |  -->  |  -1 | -->  |  -1 | -->  |  -1 |  --> |  -1 |  -->  |  -1 |
--  | Z   |       | Z   |      | Z   |      | Z   |      | Z   |       | Z   |
--  -------       -------      -------      -------      -------       -------
--    |              |            |            |            |             | 
--   \|/            \|/          \|/          \|/          \|/           \|/
--                         |
--     --------------    -----    ---------------           ---------------
--                         |
--                                                                    |  
--                            ----------------------------------    -----    ----- 
--                                                                    |   
--                                                                   
-------------------------------------------------------------------------------
--
-- Revision History: 
-- 2007.06.01 first revision. The coefficients are copied from Quartus IP core
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity NoiseFilterD is 
  port(
    aReset : in std_logic;
    Clk    : in std_logic;
    
    cDin   : in std_logic_vector(7 downto 0);
    cDout  : out std_logic_vector(7 downto 0));
end NoiseFilterD;

architecture rtl of NoiseFilterD is

  constant kNumCoes : positive := 57;
  
  type IntegerArray is array (natural range <>) of integer;
  constant kCoe : IntegerArray(kNumCoes-1 downto 0) := (
                                     -5,   6,    10,   0,    -16,  -11,  16,   31, 
                                     -1,   -48,  -34,  45,   84,   -2,   -120, -84, 
                                     105,  193,  -4,   -272, -194, 241,  463,  -5,
                                     -742, -618, 952,  3092, 4095, 3092, 952,  -618,
                                     -742, -5,   463,  241,  -194, -272, -4,   193,
                                     105,  -84,  -120, -2,   84,   45,   -34,  -48,
                                     -1,   31,   16,   -11,  -16,  0,    10,   6, 
                                     -5);
                                                
  type SignedArray is array (natural range <>) of signed(7 downto 0);
  signal cDelayData : SignedArray(kNumCoes-1 downto 0);   
  
  type ProdArray is array (natural range <>) of signed(20 downto 0);
  signal cProd : ProdArray(kNumCoes-1 downto 0);
  
  type SumArray is array (natural range <>) of signed(22 downto 0);
  signal cSumL1 : SumArray(13 downto 0);
  signal cSumL2 : SumArray(3 downto 0);
  signal cSumL3 : SumArray(0 downto 0);
                                                
begin                                           
  
  cDout <= std_logic_vector(cSumL3(0)(20 downto 13));
                                                
  -- Input data delay chain
  process(aReset, Clk)                          
  begin                                         
    if aReset='1' then                          
      for i in 0 to kNumCoes-1 loop                      
        cDelayData(i) <= (others => '0');       
      end loop;                                 
    elsif rising_edge(Clk) then                 
      cDelayData(0) <= signed(cDin);                 
      for i in 1 to kNumCoes-1 loop                      
        cDelayData(i) <= cDelayData(i-1);       
      end loop;                                 
    end if;                                     
  end process;                                  
                                                
  -- Calculate product of each tap
  process(aReset, Clk)                          
  begin                                         
    if aReset='1' then    
      for i in 0 to kNumCoes-1 loop                      
        cProd(i) <= (others => '0');       
      end loop;                                 
    elsif rising_edge(Clk) then                 
      for i in 0 to kNumCoes-1 loop                      
        cProd(i) <= cDelayData(i) * to_signed(kCoe(i), 13);       
      end loop;                                 
    end if;                                     
  end process;        
  
  -- calculate the first level sum
  process(aReset, Clk)
  begin
    if aReset='1' then
      for i in 0 to 13 loop
        cSumL1(i) <= (others => '0');
      end loop;
    elsif rising_edge(Clk) then
      for i in 0 to 13 loop
        cSumL1(i) <= resize(cProd(i*4), 23) + resize(cProd(i*4+1), 23) + 
                     resize(cProd(i*4+2), 23) + resize(cProd(i*4+3), 23);
      end loop;
    end if;
  end process;                          
  
  -- calculate the second level sum
  process(aReset, Clk)
  begin
    if aReset='1' then
      for i in 0 to 3 loop
        cSumL2(i) <= (others => '0');
      end loop;
    elsif rising_edge(Clk) then
      for i in 0 to 2 loop
        cSumL2(i) <= cSumL1(i*4) + cSumL1(i*4+1) + cSumL1(i*4+2) + cSumL1(i*4+3);
      end loop;
      cSumL2(3) <= cSumL1(12) + cSumL1(13);
    end if;
  end process;                          
  
  -- calculate the third level sum
  process(aReset, Clk)
  begin
    if aReset='1' then
      cSumL3(0) <= (others => '0');
    elsif rising_edge(Clk) then
      cSumL3(0) <= cSumL2(0) + cSumL2(1) + cSumL2(2) + cSumL2(3);
    end if;
  end process;                          
                                                
end rtl;
