-------------------------------------------------------------------------------
--
-- File: NoiseFilterT.vhd
-- Author: William Zhang 
-- Original Project: FPGA Benchmark
-- Date: 2007.06.01
--
-------------------------------------------------------------------------------
--
-- (c) 2007 Copyright Broadband Transmission Lab
-- All Rights Reserved
-- EE Dept. at Tsinghua Univ.
--
-------------------------------------------------------------------------------
--
-- Purpose: 
-- Filter the out band noise. The filter uses the transposed structure. The DC gain 
-- is 1.
-- 
--    |              |            |            |            |             | 
--   \|/            \|/          \|/          \|/          \|/           \|/
--
--  -------       -------      -------      -------      -------       -------
--  |  -1 |  -->  |  -1 | -->  |  -1 | -->  |  -1 |  --> |  -1 |  -->  |  -1 |
--  | Z   |       | Z   |      | Z   |      | Z   |      | Z   |       | Z   |
--  -------       -------      -------      -------      -------       -------
--    |                                        |            |
--    |                                        |            |
--    |                    |                   |            |
--     --------------    -----    ---------------           ---------------
--                         |
--                                                                    |  
--                            ----------------------------------    -----    ----- 
--                                                                    |   
--                                                                   
-------------------------------------------------------------------------------
--
-- Revision History: 
-- 2007.06.01 first revision. The coefficients are copied from Quartus IP core
-- 2007.11.30 change the output bits to 8 (20 downto 13)
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity NoiseFilterT is 
  port
  (aReset : in std_logic;
   Clk    : in std_logic;
   
   cDin   : in std_logic_vector(7 downto 0);
   cDout  : out std_logic_vector(7 downto 0));
end NoiseFilterT;

architecture rtl of NoiseFilterT is

  constant kNumCoes : positive := 57;
  
  type IntegerArray is array (natural range <>) of integer;
  constant kCoe : IntegerArray(kNumCoes-1 downto 0) := (
                                     -5,   6,    10,   0,    -16,  -11,  16,   31, 
                                     -1,   -48,  -34,  45,   84,   -2,   -120, -84, 
                                     105,  193,  -4,   -272, -194, 241,  463,  -5,
                                     -742, -618, 952,  3092, 4095, 3092, 952,  -618,
                                     -742, -5,   463,  241,  -194, -272, -4,   193,
                                     105,  -84,  -120, -2,   84,   45,   -34,  -48,
                                     -1,   31,   16,   -11,  -16,  0,    10,   6, 
                                     -5);

  type ProdArray is array (natural range <>) of signed(20 downto 0);
  signal cProd : ProdArray(kNumCoes-1 downto 0);
  
  type SumArray is array (natural range <>) of signed(22 downto 0);
  signal cSum : SumArray(kNumCoes-1 downto 0);
  
  signal cDin_ms, cDinDbSync : signed(7 downto 0);
  
begin
  
  cDout <= std_logic_vector(cSum(kNumCoes-1)(20 downto 13));
  
  process(aReset, Clk)
  begin
    if aReset='1' then
      cDin_ms    <= (others => '0');
      cDinDbSync <= (others => '0');
    elsif rising_edge(Clk) then
      cDin_ms    <= signed(cDin);
      cDinDbSync <= cDin_ms;
    end if;
  end process;
  
  -- multiply the input data and 
  process(aReset, Clk)
  begin
    if aReset='1' then
      for i in 0 to 56 loop
        cProd(i) <= (others => '0');
      end loop;
    elsif rising_edge(Clk) then
      for i in 0 to 56 loop
        cProd(i) <= cDinDbSync * to_signed(kCoe(56-i), 13);
      end loop;
    end if;
  end process;
  
  -- Add the products
  process(aReset, Clk)
  begin
    if aReset='1' then
      for i in 0 to kNumCoes-1 loop
        cSum(i) <= (others => '0');
      end loop;
    elsif rising_edge(Clk) then
      cSum(0) <= resize(cProd(0), 23);
      for i in 1 to kNumCoes-1 loop
        cSum(i) <= cSum(i-1) + resize(cProd(i), 23);
      end loop;
    end if;
  end process;
    
end rtl;
